# coding: GBK

to_lnglat_dict = {
    '¾­¶È': 'lng',
    'Î³¶È': 'lat',
    'lon': 'lng',
    'lng_WGS': 'lng',
    'lat_WGS': 'lat',
    'lon_WGS': 'lng',
    'longitude': 'lng',
    'latitude': 'lat',
    'geom': 'geometry'}
