# encoding:GBK

skew_skurt = '''偏度（Skewness）

用来描述数据分布的对称性，正态分布的偏度为0。
计算数据样本的偏度，当偏度<0时，称为负偏，数据出现左侧长尾；
当偏度>0时，称为正偏，数据出现右侧长尾；当偏度为0时，表示数据相对均匀的分布在平均值两侧，
不一定是绝对的对称分布，此时要与正态分布偏度为0的情况进行区分。

当偏度绝对值过大时，长尾的一侧出现极端值的可能性较高。


峰度(Kurtosis)

用来描述数据分布陡峭或是平滑的情况。正态分布的峰度为3，
峰度越大，代表分布越陡峭，尾部越厚；
峰度越小，分布越平滑。
很多情况下，为方便计算，将峰度值－3，因此正态分布的峰度变为0，方便比较。

在方差相同的情况下，峰度越大，存在极端值的可能性越高。
'''

class_dic = {'count': '计数',
             'mean': '均值',
             'std': '标准差',
             'min': '最小值',
             'max': '最大值',
             '50%': '中位数'}
